@@ -135,6 +135,14 @@ static const guint8 underline_offsets[] = {
   0, 5, 12, 19, 23
 };
 
+static const char text_position[] =
+  "baseline\0"
+  "super\0"
+  "sub\0";
+static const guint8 text_position_offsets[] = {
+  0, 9, 15,
+};
+
 static void atk_text_base_init (AtkTextIface *class);
 
 static void atk_text_real_get_range_extents  (AtkText          *text,
@@ -1031,6 +1039,22 @@ atk_text_set_selection (AtkText *text,
  *
  * Sets the caret (cursor) position to the specified @offset.
  *
+ * In the case of rich-text content, this method should either grab focus
+ * or move the sequential focus navigation starting point (if the application
+ * supports this concept) as if the user had clicked on the new caret position.
+ * Typically, this means that the target of this operation is the node containing
+ * the new caret position or one of its ancestors. In other words, after this
+ * method is called, if the user advances focus, it should move to the first
+ * focusable node following the new caret position.
+ *
+ * Calling this method should also scroll the application viewport in a way
+ * that matches the behavior of the application's typical caret motion or tab
+ * navigation as closely as possible. This also means that if the application's
+ * caret motion or focus navigation does not trigger a scroll operation, this
+ * method should not trigger one either. If the application does not have a caret
+ * motion or focus navigation operation, this method should try to scroll the new
+ * caret position into view while minimizing unnecessary scroll motion.
+ *
  * Returns: %TRUE if successful, %FALSE otherwise.
  **/
 gboolean
@@ -1319,6 +1343,9 @@ atk_text_attribute_get_value (AtkTextAttribute attr,
     case ATK_TEXT_ATTR_STYLE:
       g_assert (index >= 0 && index < G_N_ELEMENTS (style_offsets));
       return style + style_offsets[index];
+    case ATK_TEXT_ATTR_TEXT_POSITION:
+      g_assert (index >= 0 && index < G_N_ELEMENTS (text_position_offsets));
+      return text_position + text_position_offsets[index];
     default:
       return NULL;
    }
